
// BITS CODE VERSION 1.2
// Craig Lee



/*
  Low Fi Module using Teensy 3.2 and Arduino

  Code and matching hardware Craig Lee
  clsound.com

*/


#include <ADC.h>
#include <DMAChannel.h>
#include "pdb.h"

DMAChannel dma(false);

//------------

/*
   SET  bitMode CONSTANT TO CONTROL BIT MODE OPERATION

   bitMode constant sets the way the Bits control works
   Set to 1,2, or 3

   1 = Bipolar control
   tun CW of 12:00 for continuous mode
   turn CCW of 12:00 for stepped mode

   2 = Continuous mode
   number of bits is continuously variable

   3 = Stepped mode
   number of bits is stepped switching from 12 to 1

   UN-COMMENT THE LINE BELOW FOR THE BIT CONTROL FUNCTION YOU WANT
*/


//const int bitMode = 1; // bipolar control
const int bitMode = 2; //continuous control
//const int bitMode = 3; //stepped control

//------------




// pin assignments
const int rangeSwitch = 7;
const int audioIN = A10;
const int bitsCV = A6;
const int rateCV = A7;
const int effectCV = A8;
const int rateLED = 9;
const int overLED = 10;
const int underLED = 11;

// variables
volatile unsigned int adcIN;
volatile unsigned int signalInternal;
volatile unsigned int rateCVIN = 0;
volatile unsigned int depthCVIN = 0;
volatile unsigned int effectCVIN = 0;
volatile unsigned int CVsource = 1;
volatile unsigned int audioOut;

unsigned int bitDepth;
unsigned long bitRate;
unsigned int effectSelect = 0;
boolean rangeSwState;
boolean over;
boolean under;
unsigned long overCL;
unsigned long underCL;

unsigned long fxAccum;
unsigned long bitAccum;
unsigned long rateAccum;
unsigned long audioAccum;
unsigned long signalLong;
unsigned long sampleTime;
unsigned long runningTime;
unsigned long sampleTimeCV;
unsigned int sampleClock;
unsigned int sampleCV;
unsigned int cvSampleRate;
volatile unsigned int dacOut[1];

unsigned int targetFX;     // : high 3 bits of effectCVIN
unsigned int morphTo;      // = lower 9 bits of effectCVIN
unsigned int morphFrom;    // = 511 - morphHigh
unsigned int signalTemp;   // temp holder
unsigned int morphSigF;    // = effect active
unsigned int morphSigT;    // = effect going to
unsigned int b;
unsigned int bitCVVAL;


//-----------------------------------

ADC *adc = new ADC(); // adc object


void setup() {

  setupPins();

  setupDACADC();


 // Serial.begin(9600); // for debugging

  //  launch ADC
  adc->startContinuous(audioIN, ADC_1);

  runningTime = micros();

}


//-------------------------------------------------------


void loop()
{


  //Use switch to set range
  rangeSwState = digitalReadFast(rangeSwitch);
  bitRateMath();


  //  ----------FILTER CVs------------ -

  // filter bits CV
  bitAccum = bitAccum - (bitAccum >> 4) + depthCVIN;
  depthCVIN = bitAccum >> 4;

  // filter effect CV
  fxAccum = fxAccum - (fxAccum >> 4) + effectCVIN;
  effectCVIN = fxAccum >> 4;

  // filter rate CV
  rateAccum = rateAccum - (rateAccum >> 4) + bitRate;
  bitRate = rateAccum >> 4;
  sampleClock = bitRate >> 1;

  //-----------------------------------

  // Get Audio ADC result
  signalInternal = ((uint16_t)adc->analogReadContinuous(ADC_1) >> 4); //trasfer ADC_1 reading to variable as 12 bit
  runningTime = micros(); // "time stamp" reading


  // -----------------CLIP INDICATORS-------------

  if (signalInternal > 4093)
  {
    digitalWriteFast (overLED, HIGH);
  }
  else
  {
    digitalWriteFast (overLED, LOW);
  }
  if (signalInternal < 10)
  {
    digitalWriteFast (underLED, HIGH);
  }
  else
  {
    digitalWriteFast (underLED, LOW);
  }

  // ---------Do EFFECTS-------------

  // Do Morph
  setMorph();

  // Do Switch
  //setSwitch();

  // select bit reduction mode
  switch (bitMode)
  {
    case 1: // bipolar
      if (depthCVIN < 2030)
      {
        // bitDepth = (2030 - depthCVIN)  / 170;
        bitCVVAL = (2030 - depthCVIN) << 1;
        steppedBits();
        truncate();
      }
      else
      {
        // bitDepth = (depthCVIN - 2030) + 40;
        bitCVVAL = (depthCVIN - 2030) << 1;
        continuousBits();
        truncateINT();
      }
      break;

    case 2: // continuous
      bitCVVAL = depthCVIN;
      continuousBits();
      truncateINT();
      break;

    case 3: // stepped
      bitCVVAL = depthCVIN;
      steppedBits();
      truncate();
      break;
  }


  // --------SAMPLE AND HOLD Write to DAC
  if (bitRate > 15)
  {

    if (runningTime - sampleTime > bitRate)
    {
      dacOut[0] = signalInternal;
      sampleTime = micros(); // start counting up to next sample
      //digitalWriteFast(15, !digitalReadFast(15));
    }
  }
  else
  {
    dacOut[0] = signalInternal; //
   // digitalWriteFast(15, !digitalReadFast(15));
  }
  //------------------------

  //  LED FOR CLOCK AND SR OUT
  if (rangeSwState == 0)
  {
    if (runningTime - sampleTime < sampleClock)
    {
      digitalWriteFast (rateLED, HIGH);
      //digitalWriteFast(13, HIGH); //onboard LED
    }
    else
    {
      digitalWriteFast (rateLED, LOW);
      //digitalWriteFast(13, LOW);
    }
  }
  else
  {
    digitalWriteFast (rateLED, LOW);
    digitalWriteFast(13, LOW);
  }


  // SAMPLE CVs
  if ( runningTime -   sampleTimeCV > cvSampleRate)
  {
    launchRead();
    sampleTimeCV = micros();
  }

}

// END OF MAIN LOOP


//----Subroutines------------------------------



void launchRead()
{

  digitalWriteFast(14, !digitalReadFast(14));

  switch (CVsource)
  {
    case 1: // rate in
      rateCVIN = (uint16_t)adc->readSingle( ADC_0);
      adc->startSingleRead(bitsCV, ADC_0);
      CVsource = 2;
      break;

    case 2: // bit depth CV
      depthCVIN = (uint16_t)adc->readSingle( ADC_0);
      adc->startSingleRead(effectCV, ADC_0);
      CVsource = 3;

 /*     Serial.print("depthCVIN - ");
      Serial.print(depthCVIN);
      Serial.print(" : Bit Depth - ");
      Serial.print(bitDepth);
      Serial.print(" : Bits - ");
      Serial.print(4096/bitDepth);
      Serial.println();
*/
      break;

    case 3: // Effect
      effectCVIN = (uint16_t)adc->readSingle( ADC_0);
      adc->startSingleRead(rateCV, ADC_0);
      CVsource = 1;
      break;
  }


}

//------------------------------------

// stepped truncate
void truncate()
{

  switch (bitDepth)
  {
    case 1:
      signalInternal &= 4094;
      break;
    case 2:
      signalInternal &= 4092;
      break;
    case 3:
      signalInternal &= 4088;
      break;
    case 4:
      signalInternal &= 4080;
    case 5:
      signalInternal &= 4064;
      break;
    case 6:
      signalInternal &= 4032;
      break;
    case 7:
      signalInternal &= 3968;
      break;
    case 8:
      signalInternal &= 3840;
      break;
    case 9:
      signalInternal &= 3584;
      break;
    case 10:
      signalInternal &= 3072;
      break;
    case 11:
      signalInternal &= 2048;
      break;
    default:
      //signalInternal &= 4095;
      break;
  }
}
// Continuous truncate

void truncateINT()
{
  signalInternal = signalInternal / bitDepth;
  signalInternal = signalInternal * bitDepth;
}



// -----------END-----------------
